package eu.kanade.tachiyomi.ui.security

import android.os.Bundle
import androidx.biometric.BiometricPrompt
import eu.kanade.tachiyomi.R
import eu.kanade.tachiyomi.ui.base.activity.BaseActivity
import java.util.Date
import java.util.concurrent.ExecutorService
import java.util.concurrent.Executors

class BiometricActivity : BaseActivity() {
    private val executor: ExecutorService = Executors.newSingleThreadExecutor()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val fromSearch = intent.getBooleanExtra("fromSearch", false)
        val biometricPrompt = BiometricPrompt(this, executor, object : BiometricPrompt
        .AuthenticationCallback() {

            override fun onAuthenticationError(errorCode: Int, errString: CharSequence) {
                super.onAuthenticationError(errorCode, errString)
                if (fromSearch) finish()
                else finishAffinity()
            }

            override fun onAuthenticationSucceeded(result: BiometricPrompt.AuthenticationResult) {
                super.onAuthenticationSucceeded(result)
                SecureActivityDelegate.locked = false
                preferences.lastUnlock().set(Date().time)
                finish()
            }
        })

        val promptInfo = BiometricPrompt.PromptInfo.Builder()
            .setTitle(getString(R.string.unlock_library))
            .setNegativeButtonText(getString(android.R.string.cancel))
            .build()

        biometricPrompt.authenticate(promptInfo)
    }

}